"use strict";
Object.defineProperty(exports, "__esModule", { value: true });
const registry_1 = require("../../utils/registry");
const registryData = require("../data/registry.json");
const errors_1 = require("../../errors");
const path_1 = require("../../utils/path");
const forceIgnore_1 = require("../forceIgnore");
const path_2 = require("path");
const treeContainers_1 = require("../treeContainers");
const sourceComponent_1 = require("../sourceComponent");
class BaseSourceAdapter {
    constructor(type, registry = registryData, forceIgnore = new forceIgnore_1.ForceIgnore(), tree = new treeContainers_1.NodeFSTreeContainer()) {
        /**
         * Whether or not an adapter should expect a component to be in its own, self-named
         * folder, including its root metadata xml file.
         */
        this.ownFolder = false;
        this.type = type;
        this.registry = registry;
        this.forceIgnore = forceIgnore;
        this.tree = tree;
    }
    getComponent(path) {
        let rootMetadata = this.parseAsRootMetadataXml(path);
        if (!rootMetadata) {
            const rootMetadataPath = this.getRootMetadataXmlPath(path);
            if (!rootMetadataPath) {
                throw new errors_1.RegistryError('error_missing_metadata_xml', [path, this.type.name]);
            }
            rootMetadata = registry_1.parseMetadataXml(rootMetadataPath);
        }
        if (this.forceIgnore.denies(rootMetadata.path)) {
            throw new errors_1.UnexpectedForceIgnore('error_no_metadata_xml_ignore', [rootMetadata.path, path]);
        }
        const componentName = this.type.inFolder
            ? `${path_1.parentName(rootMetadata.path)}/${rootMetadata.fullName}`
            : rootMetadata.fullName;
        const component = new sourceComponent_1.SourceComponent({
            name: componentName,
            type: this.type,
            xml: rootMetadata.path,
        }, this.tree, this.forceIgnore);
        return this.populate(component, path);
    }
    /**
     * If the path given to `getComponent` is the root metadata xml file for a component,
     * parse the name and return it. This is an optimization to not make a child adapter do
     * anymore work to find it.
     *
     * @param path File path of a metadata component
     */
    parseAsRootMetadataXml(path) {
        let isRootMetadataXml = false;
        const metaXml = registry_1.parseMetadataXml(path);
        if (metaXml) {
            const requireStrictParent = !!this.registry.mixedContent[this.type.directoryName];
            if (requireStrictParent) {
                const parentPath = path_2.dirname(path);
                const typeDirName = path_2.basename(this.type.inFolder ? path_2.dirname(parentPath) : parentPath);
                const nameMatchesParent = path_2.basename(parentPath) === metaXml.fullName;
                const inTypeDir = typeDirName === this.type.directoryName;
                // if the parent folder name matches the fullName OR parent folder name is
                // metadata type's directory name, it's a root metadata xml.
                isRootMetadataXml = nameMatchesParent || inTypeDir;
            }
            else {
                isRootMetadataXml = true;
            }
        }
        return isRootMetadataXml ? metaXml : undefined;
    }
}
exports.BaseSourceAdapter = BaseSourceAdapter;
//# sourceMappingURL=baseSourceAdapter.js.map