"use strict";
/*
 * Copyright (c) 2020, salesforce.com, inc.
 * All rights reserved.
 * Licensed under the BSD 3-Clause license.
 * For full license text, see LICENSE.txt file in the repo root or https://opensource.org/licenses/BSD-3-Clause
 */
Object.defineProperty(exports, "__esModule", { value: true });
const registryAccess_1 = require("./registryAccess");
const errors_1 = require("../errors");
const fs_1 = require("fs");
class ManifestGenerator {
    constructor(registryAccess = new registryAccess_1.RegistryAccess()) {
        this.xmlDef = '<?xml version="1.0" encoding="UTF-8"?>\n';
        this.packageModuleStart = '<Package xmlns="http://soap.sforce.com/2006/04/metadata">\n';
        this.packageModuleEnd = '</Package>';
        this.registryAccess = registryAccess;
    }
    createManifestFromPath(sourcePath, outputPath) {
        try {
            const mdComponents = this.registryAccess.getComponentsFromPath(sourcePath);
            fs_1.writeFileSync(outputPath, this.createManifest(mdComponents));
        }
        catch (err) {
            throw new errors_1.RegistryError('error_on_manifest_creation', [sourcePath, err]);
        }
    }
    createManifest(components, apiVersion = this.registryAccess.getApiVersion()) {
        let output = this.xmlDef.concat(this.packageModuleStart);
        const metadataMap = this.createMetadataMap(components);
        for (const metadataType of metadataMap.keys()) {
            output = output.concat('  <types>\n');
            for (const metadataName of metadataMap.get(metadataType)) {
                output = output.concat(`    <members>${metadataName}</members>\n`);
            }
            output = output.concat(`    <name>${metadataType}</name>\n`);
            output = output.concat('  </types>\n');
        }
        output = output.concat(`  <version>${apiVersion}</version>\n`, this.packageModuleEnd);
        return output;
    }
    createMetadataMap(components) {
        const metadataMap = new Map();
        for (const component of components) {
            const metadataType = this.registryAccess.getTypeFromName(component.type.name).name;
            const metadataName = component.fullName;
            if (metadataMap.has(metadataType)) {
                const metadataNames = metadataMap.get(metadataType);
                metadataNames.add(metadataName);
                metadataMap.set(metadataType, metadataNames);
            }
            else {
                const metadataNames = new Set();
                metadataNames.add(metadataName);
                metadataMap.set(metadataType, metadataNames);
            }
        }
        return metadataMap;
    }
}
exports.ManifestGenerator = ManifestGenerator;
//# sourceMappingURL=manifestGenerator.js.map