"use strict";
Object.defineProperty(exports, "__esModule", { value: true });
const path_1 = require("path");
const forceIgnore_1 = require("./forceIgnore");
const registry_1 = require("../utils/registry");
const utils_1 = require("../utils");
const treeContainers_1 = require("./treeContainers");
/**
 * Representation of a MetadataComponent in a file tree.
 */
class SourceComponent {
    constructor(props, tree = new treeContainers_1.NodeFSTreeContainer(), forceIgnore = new forceIgnore_1.ForceIgnore()) {
        this.name = props.name;
        this.type = props.type;
        this.xml = props.xml;
        this.parent = props.parent;
        this.content = props.content;
        this._tree = tree;
        this.forceIgnore = forceIgnore;
    }
    static createVirtualComponent(props, fs, forceIgnore) {
        const tree = new treeContainers_1.VirtualTreeContainer(fs);
        return new SourceComponent(props, tree, forceIgnore);
    }
    walkContent() {
        const sources = [];
        if (this.content) {
            for (const fsPath of this.walk(this.content)) {
                if (fsPath !== this.xml) {
                    sources.push(fsPath);
                }
            }
        }
        return sources;
    }
    getChildren() {
        const parentPath = path_1.dirname(this.xml);
        return this.getChildrenInternal(parentPath);
    }
    getChildrenInternal(dirPath) {
        const children = [];
        for (const fsPath of this.walk(dirPath)) {
            const childXml = registry_1.parseMetadataXml(fsPath);
            const fileIsRootXml = childXml.suffix === this.type.suffix;
            if (childXml && !fileIsRootXml) {
                // TODO: Log warning if missing child type definition
                const childTypeId = this.type.children.suffixes[childXml.suffix];
                const childComponent = new SourceComponent({
                    name: utils_1.baseName(fsPath),
                    type: this.type.children.types[childTypeId],
                    xml: fsPath,
                    parent: this,
                }, this._tree, this.forceIgnore);
                children.push(childComponent);
            }
        }
        return children;
    }
    *walk(fsPath) {
        if (!this._tree.isDirectory(fsPath)) {
            yield fsPath;
        }
        else {
            for (const child of this._tree.readDirectory(fsPath)) {
                const childPath = path_1.join(fsPath, child);
                if (this.forceIgnore.denies(childPath)) {
                    continue;
                }
                else if (this._tree.isDirectory(childPath)) {
                    yield* this.walk(childPath);
                }
                else {
                    yield childPath;
                }
            }
        }
    }
    get fullName() {
        return `${this.parent ? `${this.parent.fullName}.` : ''}${this.name}`;
    }
}
exports.SourceComponent = SourceComponent;
//# sourceMappingURL=sourceComponent.js.map