import { Connection } from '@salesforce/core';
import { SourcePath } from './common';
import { RegistryAccess, SourceComponent } from '../metadata-registry';
export declare type RecordId = string;
export declare type ComponentDeployment = {
    id?: string;
    component: SourceComponent;
    status: ComponentStatus;
    diagnostics: ComponentDiagnostic[];
};
export declare type ComponentDiagnostic = {
    lineNumber?: number;
    columnNumber?: number;
    filePath?: SourcePath;
    message: string;
    type: 'Warning' | 'Error';
};
/**
 * Possible statuses of a component being deployed.
 */
export declare enum ComponentStatus {
    Created = "Created",
    Changed = "Changed",
    Unchanged = "Unchanged",
    Deleted = "Deleted",
    Failed = "Failed"
}
interface SourceApiResult {
    success: boolean;
}
export interface SourceDeployResult extends SourceApiResult {
    id: RecordId;
    components?: ComponentDeployment[];
    status: DeployStatus | ToolingDeployStatus;
}
/**
 * Raw response returned from a checkDeployStatus call to the Metadata API
 */
export declare type DeployResult = {
    id: string;
    canceledBy?: string;
    canceledByName?: string;
    checkOnly: boolean;
    completedDate?: string;
    createdBy: string;
    createdByName: string;
    createdDate: string;
    details: DeployDetails;
    done: boolean;
    errorMessage?: string;
    errorStatusCode?: string;
    ignoreWarnings: boolean;
    lastModifiedDate: string;
    numberComponentErrors: number;
    numberComponentsDeployed: number;
    numberComponentsTotal: number;
    numberTestErrors: number;
    numberTestsCompleted: number;
    numberTestsTotal: number;
    runTestsEnabled: boolean;
    rollbackOnError: boolean;
    startDate?: string;
    stateDetail?: string;
    status: DeployStatus;
    success: boolean;
};
/**
 * Possible statuses of a metadata deploy operation.
 */
export declare enum DeployStatus {
    Pending = "Pending",
    InProgress = "InProgress",
    Succeeded = "Succeeded",
    SucceededPartial = "SucceededPartial",
    Failed = "Failed",
    Canceling = "Canceling",
    Canceled = "Canceled"
}
export declare type DeployDetails = {
    componentFailures?: DeployMessage[];
    componentSuccesses?: DeployMessage[];
};
declare type BooleanString = 'true' | 'false' | true | false;
export declare type DeployMessage = {
    changed: BooleanString;
    columnNumber?: string;
    componentType?: string;
    created: BooleanString;
    createdDate: string;
    deleted: BooleanString;
    fileName: string;
    fullName: string;
    id?: string;
    lineNumber?: string;
    problem?: string;
    problemType?: 'Warning' | 'Error';
    success: BooleanString;
};
export declare type ContainerAsyncRequest = {
    Id: RecordId;
    DeployDetails?: DeployDetails;
    ErrorMsg?: string;
    State?: ToolingDeployStatus;
};
export declare const enum ToolingDeployStatus {
    Queued = "Queued",
    Invalidated = "Invalidated",
    Error = "Error",
    Aborted = "Aborted",
    Completed = "Completed",
    Failed = "Failed",
    CompletedPartial = "CompletedPartial"
}
export interface DeployRetrieveClient {
    /**
     * Retrieve metadata components and wait for the result.
     *
     * @param options Specify `components`, `output` and other optionals
     */
    retrieve(options: RetrieveOptions): Promise<ApiResult>;
    /**
     * Infer metadata components from source paths, retrieve them, and wait for the result.
     *
     * @param options Specify `paths`, `output` and other optionals
     */
    retrieveWithPaths(options: RetrievePathOptions): Promise<ApiResult>;
    /**
     * Deploy metadata components and wait for result.
     *
     * @param filePath Paths to source files to deploy
     */
    deploy(components: SourceComponent | SourceComponent[]): Promise<SourceDeployResult>;
    /**
     * Infer metadata components from source path, deploy them, and wait for results.
     *
     * @param filePath Paths to source files to deploy
     */
    deployWithPaths(paths: SourcePath | SourcePath[]): Promise<SourceDeployResult>;
}
export declare abstract class BaseApi implements DeployRetrieveClient {
    protected connection: Connection;
    protected registry: RegistryAccess;
    constructor(connection: Connection, registry: RegistryAccess);
    /**
     * @param options Specify `paths`, `output` and other optionals
     */
    abstract retrieveWithPaths(options: RetrievePathOptions): Promise<ApiResult>;
    abstract retrieve(options: RetrieveOptions): Promise<ApiResult>;
    abstract deploy(components: SourceComponent | SourceComponent[]): Promise<SourceDeployResult>;
    abstract deployWithPaths(paths: SourcePath | SourcePath[]): Promise<SourceDeployResult>;
}
declare type CommonOptions = {
    /**
     * Set the max number of seconds to wait for the operation.
     */
    wait?: number;
    namespace?: string;
};
declare type CommonRetrieveOptions = {
    /**
     * Whether or not the files should be automatically converted to
     * [source format](https://developer.salesforce.com/docs/atlas.en-us.sfdx_dev.meta/sfdx_dev/sfdx_dev_source_file_format.htm)
     */
    convert?: boolean;
    /**
     * Whether or not existing source files should be overwritten.
     */
    overwrite?: boolean;
    /**
     * The directory to retrieve the components to.
     */
    output?: SourcePath;
};
declare type CommonPathOptions = {
    /**
     * Source paths of the files to perform the operation on.
     */
    paths: SourcePath[];
};
export declare type RetrieveOptions = CommonOptions & CommonRetrieveOptions & {
    components: SourceComponent[];
};
export declare type RetrievePathOptions = CommonOptions & CommonRetrieveOptions & CommonPathOptions;
export declare type ApiResult = {
    success: boolean;
    components: SourceComponent[];
    message?: string;
};
declare type WaitFlag = {
    wait?: number;
};
declare type NamespaceFlag = {
    namespace?: string;
};
export declare type MetadataApiDeployOptions = {
    allowMissingFiles?: boolean;
    autoUpdatePackage?: boolean;
    checkOnly?: boolean;
    ignoreWarnings?: boolean;
    performRetrieve?: boolean;
    purgeOnDelete?: boolean;
    rollbackOnError?: boolean;
    runAllTests?: boolean;
    runTests?: string[];
    singlePackage?: boolean;
};
export declare type MetadataDeployOptions = WaitFlag & {
    apiOptions?: MetadataApiDeployOptions;
};
export declare type ToolingDeployOptions = NamespaceFlag;
export declare type DeployPathOptions = CommonOptions & CommonPathOptions;
export {};
